import java.math.BigInteger;
import java.util.Random;

/**
 * Klasse RSA.
 * 
 * @author claus albowski
 * @version okt'07
 */
public class RSA
{
    private Random random;
    int bitLength;
    private BigInteger p, q, n, phi, d, e, original, chiffrat, dechiffrat;

    /**
     * Konstruktor fr Objekte der Klasse RSA.
     */
    public RSA()   {
        original = new BigInteger("123456789");
        chiffrat = new BigInteger("0");
        dechiffrat = new BigInteger("0");
        random=new Random();
        bitLength=20;
        definiereNPQPHI();
        setzeEundD();
    }
    
    /**
     * Konstruktor fr Objekte der Klasse RSA
     * mit bergebener bit - Lnge. 
     */
    public RSA(int bitLaenge)   {
        setzeOriginal("123456789123456789");
        chiffrat = new BigInteger("0");
        dechiffrat = new BigInteger("0");
        random=new Random();
        bitLength=bitLaenge;
        definiereNPQPHI();
        setzeEundD();
    }
    
    /**
     * setzt das Original als BigInteger.
     */
    public void setzeOriginal(String botschaft){
        original=new BigInteger(botschaft);
    }
    
    /**
     * setzt das Chiffrat als BigInteger.
     */
    public void setzeChiffrat(String chiffratString){
        chiffrat = new BigInteger(chiffratString);
    }
    
    /**
     * zeigt das Original als String an.
     */
    public String gibOriginal(){
        return original.toString();
    }

    /**
     * zeigt das Chiffrat als String an.
     */
    public String gibChiffrat(){
        return chiffrat.toString();
    }

    /**
     * zeigt das Dechiffrat als String an.
     */
    public String gibDechiffrat(){
        return dechiffrat.toString();
    }
    
    // bersetzungsmethoden:
    
    /**
     * gibt eine zufllige Pseudoprimzahl mit der bergebenen Bit-Lnge zurck.
     */
    private BigInteger gibZufallsPrimzahl(int bitLaenge){
        return new BigInteger(bitLaenge, 100, random);
    }
   
    /**
     * whlt zunchst zwei (Pseudo-)Primzahlen p und q, berechnet n und phi.
     */
    public void definiereNPQPHI(){
        p=gibZufallsPrimzahl(bitLength);
        q=gibZufallsPrimzahl(bitLength);
        n=p.multiply(q);
        phi=(p.subtract(BigInteger.ONE)).multiply(q.subtract(BigInteger.ONE));
    }

    
    /**
     * setzt e und dazu d.
     */
    public void setzeEundD(){
        e=gibZufallsPrimzahl(2*bitLength+1).mod(phi);
        d=e.modInverse(phi);
    }
    
    /**
     * chiffriert mit RSA
     * und setzt das chiffrat.
     */
    public BigInteger chiffriere(){
        BigInteger ergebnis = original.modPow(e, n);
        chiffrat = ergebnis;
        return ergebnis;
    }
    
    /**
     * dechiffriert mit RSA
     * und setzt das dechiffrat.
     */
    public BigInteger dechiffriere(){
        BigInteger ergebnis = chiffrat.modPow(d, n);
        dechiffrat = ergebnis;
        return ergebnis;
    }
    
}